@@ -37,8 +37,8 @@
 #define avcodec_open2(ctx,codec,data) avcodec_open(ctx,codec)
 #endif
 
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(53, 17, 0)
-#define avformat_free_context(ctx) av_close_input_file(ctx)
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(55, 16, 101)
+#define avformat_close_input(ctx) {av_close_input_file(*(ctx)); *(ctx) = NULL;}
 #endif
 
 #if LIBAVCODEC_VERSION_INT > AV_VERSION_INT(55, 28, 0)
@@ -87,16 +87,6 @@ static DB_functions_t *deadbeef;
 
 static char * exts[EXT_MAX+1] = {NULL};
 
-enum {
-    FT_ALAC = 0,
-    FT_WMA = 1,
-    FT_ATRAC3 = 2,
-    FT_VQF = 3,
-    FT_AC3 = 4,
-    FT_AMR = 5,
-    FT_UNKNOWN = 5
-};
-
 typedef struct {
     DB_fileinfo_t info;
     AVCodec *codec;
@@ -122,9 +112,8 @@ typedef struct {
 
 static DB_fileinfo_t *
 ffmpeg_open (uint32_t hints) {
-    DB_fileinfo_t *_info = malloc (sizeof (ffmpeg_info_t));
-    memset (_info, 0, sizeof (ffmpeg_info_t));
-    return _info;
+    ffmpeg_info_t *info = calloc (sizeof (ffmpeg_info_t), 1);
+    return &info->info;
 }
 
 // ensure that the buffer can contain entire frame of frame_size bytes per channel
@@ -282,8 +271,9 @@ ffmpeg_free (DB_fileinfo_t *_info) {
             avcodec_close (info->ctx);
         }
         if (info->fctx) {
-            avformat_free_context (info->fctx);
+            avformat_close_input (&info->fctx);
         }
+
         free (info);
     }
 }
@@ -716,7 +706,7 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     if (codec == NULL)
     {
         trace ("ffmpeg can't decode %s\n", fname);
-        avformat_free_context(fctx);
+        avformat_close_input (&fctx);
         return NULL;
     }
     trace ("ffmpeg can decode %s\n", fname);
@@ -724,7 +714,7 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
 
     if (avcodec_open2 (ctx, codec, NULL) < 0) {
         trace ("ffmpeg: avcodec_open2 failed\n");
-        avformat_free_context(fctx);
+        avformat_close_input (&fctx);
         return NULL;
     }
 
@@ -782,7 +772,7 @@ ffmpeg_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
 
     // free decoder
     avcodec_close (ctx);
-    avformat_free_context(fctx);
+    avformat_close_input (&fctx);
 
     DB_playItem_t *cue = deadbeef->plt_process_cue (plt, after, it, totalsamples, samplerate);
     if (cue) {
@@ -974,19 +964,19 @@ ffmpeg_read_metadata (DB_playItem_t *it) {
     if (codec == NULL)
     {
         trace ("ffmpeg can't decode %s\n", deadbeef->pl_find_meta (it, ":URI"));
-        avformat_free_context(fctx);
+        avformat_close_input (&fctx);
         return -1;
     }
     if (avcodec_open2 (ctx, codec, NULL) < 0) {
         trace ("ffmpeg: avcodec_open2 failed\n");
-        avformat_free_context(fctx);
+        avformat_close_input (&fctx);
         return -1;
     }
 
     deadbeef->pl_delete_all_meta (it);
     ffmpeg_read_metadata_internal (it, fctx);
 
-    avformat_free_context(fctx);
+    avformat_close_input (&fctx);
 
     return 0;
 }
