@@ -40,12 +40,16 @@
 #include "trkproperties.h"
 #include "callbacks.h"
 #include <sys/stat.h>
+#include "gtkui_api.h"
+#include <gio/gio.h>
 
 // disable custom title function, until we have new title formatting (0.7)
 #define DISABLE_CUSTOM_TITLE
 
 extern GtkWidget *mainwin;
 extern DB_functions_t *deadbeef;
+extern ddb_gtkui_t plugin;
+#define trace(...) { deadbeef->log_detailed (&plugin.gui.plugin, 0, __VA_ARGS__); }
 
 gboolean
 action_open_files_handler_cb (void *userdata) {
@@ -57,7 +61,7 @@ action_open_files_handler_cb (void *userdata) {
 }
 
 int
-action_open_files_handler (struct DB_plugin_action_s *action, int ctx) {
+action_open_files_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_open_files_handler_cb, NULL);
     return 0;
 }
@@ -72,7 +76,7 @@ action_add_files_handler_cb (void *user_data) {
 }
 
 int
-action_add_files_handler (struct DB_plugin_action_s *action, int ctx) {
+action_add_files_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_add_files_handler_cb, NULL);
     return 0;
 }
@@ -87,7 +91,7 @@ action_add_folders_handler_cb (void *user_data) {
 }
 
 int
-action_add_folders_handler (struct DB_plugin_action_s *action, int ctx) {
+action_add_folders_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_add_folders_handler_cb, NULL);
     return 0;
 }
@@ -99,7 +103,7 @@ action_quit_handler_cb (void *user_data) {
 }
 
 int
-action_quit_handler (DB_plugin_action_t *act, int ctx) {
+action_quit_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_quit_handler_cb, NULL);
     return 0;
 }
@@ -122,7 +126,7 @@ action_deselect_all_handler_cb (void *user_data) {
 }
 
 int
-action_deselect_all_handler (struct DB_plugin_action_s *action, int ctx) {
+action_deselect_all_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     g_idle_add (action_deselect_all_handler_cb, NULL);
     return 0;
 }
@@ -135,7 +139,7 @@ action_select_all_handler_cb (void *user_data) {
 }
 
 int
-action_select_all_handler (struct DB_plugin_action_s *action, int ctx) {
+action_select_all_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     g_idle_add (action_select_all_handler_cb, NULL);
     return 0;
 }
@@ -151,13 +155,13 @@ action_new_playlist_handler_cb (void *user_data) {
 }
 
 int
-action_new_playlist_handler (struct DB_plugin_action_s *action, int ctx) {
+action_new_playlist_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_new_playlist_handler_cb, NULL);
     return 0;
 }
 
 int
-action_remove_current_playlist_handler (struct DB_plugin_action_s *action, int ctx) {
+action_remove_current_playlist_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     int idx = deadbeef->plt_get_curr_idx ();
     if (idx != -1) {
         deadbeef->plt_remove (idx);
@@ -172,7 +176,7 @@ action_toggle_mainwin_handler_cb (void *user_data) {
 }
 
 int
-action_toggle_mainwin_handler (struct DB_plugin_action_s *action, int ctx) {
+action_toggle_mainwin_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     g_idle_add (action_toggle_mainwin_handler_cb, NULL);
     return 0;
 }
@@ -193,7 +197,7 @@ action_show_mainwin_handler_cb (void *user_data) {
 }
 
 int
-action_show_mainwin_handler (struct DB_plugin_action_s *action, int ctx) {
+action_show_mainwin_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     g_idle_add (action_show_mainwin_handler_cb, NULL);
     return 0;
 }
@@ -208,11 +212,13 @@ action_hide_mainwin_handler_cb (void *user_data) {
 }
 
 int
-action_hide_mainwin_handler (struct DB_plugin_action_s *action, int ctx) {
+action_hide_mainwin_handler (struct DB_plugin_action_s *action, ddb_action_context_t ctx) {
     g_idle_add (action_hide_mainwin_handler_cb, NULL);
     return 0;
 }
 
+// FIXME: this functionality was supposed to be enabled in 0.7.0, but was forgotten
+#ifndef DISABLE_CUSTOM_TITLE
 static void
 on_toggle_set_custom_title (GtkToggleButton *togglebutton, gpointer user_data) {
     gboolean active = gtk_toggle_button_get_active (togglebutton);
@@ -223,6 +229,7 @@ on_toggle_set_custom_title (GtkToggleButton *togglebutton, gpointer user_data) {
 
     deadbeef->conf_save ();
 }
+#endif
 
 gboolean
 action_add_location_handler_cb (void *user_data) {
@@ -265,6 +272,8 @@ action_add_location_handler_cb (void *user_data) {
                     if (it && deadbeef->conf_get_int ("gtkui.location_set_custom_title", 0)) {
                         deadbeef->pl_replace_meta (it, ":CUSTOM_TITLE", gtk_entry_get_text (GTK_ENTRY (ct)));
                     }
+#else
+#   pragma unused(it)
 #endif
                     if (tail) {
                         deadbeef->pl_item_unref (tail);
@@ -283,7 +292,7 @@ action_add_location_handler_cb (void *user_data) {
 }
 
 int
-action_add_location_handler (DB_plugin_action_t *act, int ctx) {
+action_add_location_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_add_location_handler_cb, NULL);
     return 0;
 }
@@ -299,13 +308,13 @@ action_show_help_handler_cb (void *user_data) {
 }
 
 int
-action_show_help_handler (DB_plugin_action_t *act, int ctx) {
+action_show_help_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_show_help_handler_cb, NULL);
     return 0;
 }
 
 int
-action_remove_from_playlist_handler (DB_plugin_action_t *act, int ctx) {
+action_remove_from_playlist_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     if (ctx == DDB_ACTION_CTX_SELECTION) {
         ddb_playlist_t *plt = deadbeef->plt_get_curr ();
         if (plt) {
@@ -334,7 +343,6 @@ action_remove_from_playlist_handler (DB_plugin_action_t *act, int ctx) {
         deadbeef->plt_unref (plt_curr);
     }
     else if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
-        int success = 0;
         deadbeef->pl_lock ();
         DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
         if (it) {
@@ -343,7 +351,7 @@ action_remove_from_playlist_handler (DB_plugin_action_t *act, int ctx) {
                 int idx = deadbeef->plt_get_item_idx (plt, it, PL_MAIN);
                 if (idx != -1) {
                     deadbeef->plt_remove_item (plt, it);
-                    deadbeef->pl_save_current ();
+                    deadbeef->plt_save_config (plt);
                     deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
                 }
                 deadbeef->plt_unref (plt);
@@ -355,25 +363,82 @@ action_remove_from_playlist_handler (DB_plugin_action_t *act, int ctx) {
     return 0;
 }
 
-void
-delete_and_remove_track (const char *uri, ddb_playlist_t *plt, ddb_playItem_t *it) {
-    int res = unlink (uri);
+static void
+_remove_file_from_all_playlists (const char *search_uri) {
+    // The caller is responsible for pl_lock
+    int n = deadbeef->plt_get_count ();
+    for (int i = 0; i < n; ++i) {
+        ddb_playlist_t *plt = deadbeef->plt_get_for_idx (i);
+        DB_playItem_t *it = deadbeef->plt_get_first (plt, PL_MAIN);
+        while (it) {
+            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
+            const char *uri = deadbeef->pl_find_meta (it, ":URI");
+            if (strcmp (uri, search_uri) == 0) {
+                deadbeef->plt_remove_item (plt, it);
+            }
+            deadbeef->pl_item_unref (it);
+            it = next;
+        }
+
+        deadbeef->plt_unref (plt);
+    }
+}
+
+static void
+_delete_and_remove_track_from_all_playlists (const char *uri, ddb_playlist_t *plt, ddb_playItem_t *it) {
+
+    if (deadbeef->conf_get_int ("gtkui.move_to_trash", 1)) {
+        GFile *file = g_file_new_for_path (uri);
+        g_file_trash (file, NULL, NULL);
+        g_object_unref (file);
+    } else {
+        (void)unlink (uri);
+    }
 
     // check if file exists
     struct stat buf;
     memset (&buf, 0, sizeof (buf));
     int stat_res = stat (uri, &buf);
+    
     if (stat_res != 0) {
-        deadbeef->plt_remove_item (plt, it);
+        _remove_file_from_all_playlists (uri);
+    } else {
+        trace ("Failed to delete file: %s\n", uri);
     }
 }
 
 gboolean
 action_delete_from_disk_handler_cb (void *data) {
-    int ctx = (intptr_t)data;
+    ddb_playlist_t *plt = deadbeef->plt_get_curr ();
+    if (!plt) {
+        return FALSE;
+    }
+    
+    ddb_action_context_t ctx = (int)(intptr_t)data;
     if (deadbeef->conf_get_int ("gtkui.delete_files_ask", 1)) {
+        char buf[1000];
+        const char *buf2 = deadbeef->conf_get_int ("gtkui.move_to_trash", 1) ?
+        _(" The files will be moved to trash.\n\n(This dialog can be turned off in GTKUI plugin settings)") : 
+        _(" The files will be lost.\n\n(This dialog can be turned off in GTKUI plugin settings)");
+
+        if (ctx == DDB_ACTION_CTX_SELECTION) {
+            int selected_files = deadbeef->pl_getselcount ();
+            if (selected_files == 1) {
+                snprintf(buf, sizeof (buf), _("Do you really want to delete the selected file?%s"), buf2);
+            } else {
+                snprintf(buf, sizeof (buf), _("Do you really want to delete all %d selected files?%s"), selected_files, buf2);
+            }
+        }
+        else if (ctx == DDB_ACTION_CTX_PLAYLIST) {
+            int files = deadbeef->plt_get_item_count (plt, PL_MAIN);
+            snprintf(buf, sizeof (buf), _("Do you really want to delete all %d files from the current playlist?%s"), files, buf2);
+        }
+        else if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
+            snprintf(buf, sizeof (buf), _("Do you really want to delete the currently playing file?%s"), buf2);
+        }
+
         GtkWidget *dlg = gtk_message_dialog_new (GTK_WINDOW (mainwin), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_YES_NO, _("Delete files from disk"));
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg), _("Files will be lost. Proceed?\n(This dialog can be turned off in GTKUI plugin settings)"));
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg), "%s", buf);
         gtk_window_set_title (GTK_WINDOW (dlg), _("Warning"));
 
         int response = gtk_dialog_run (GTK_DIALOG (dlg));
@@ -382,79 +447,111 @@ action_delete_from_disk_handler_cb (void *data) {
             return FALSE;
         }
     }
-
-    ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-    if (!plt) {
-        return FALSE;
-    }
     deadbeef->pl_lock ();
 
+    DB_playItem_t **tracklist = NULL;
+    unsigned trackcount = 0;
+    
+    DB_playItem_t *it_current_song = deadbeef->streamer_get_playing_track ();
+    int idx_current_song = -1;
     if (ctx == DDB_ACTION_CTX_SELECTION) {
+        unsigned selcount = deadbeef->plt_getselcount (plt);
+        tracklist = calloc (selcount, sizeof (DB_playItem_t *));
         DB_playItem_t *it = deadbeef->plt_get_first (plt, PL_MAIN);
         while (it) {
+            if (trackcount == selcount) {
+                break;
+            }
             DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
             const char *uri = deadbeef->pl_find_meta (it, ":URI");
             if (deadbeef->pl_is_selected (it) && deadbeef->is_local_file (uri)) {
-                delete_and_remove_track (uri, plt, it);
+                if (it == it_current_song) {
+                    idx_current_song = deadbeef->plt_get_item_idx (plt, it, PL_MAIN);
+                }
+                deadbeef->pl_item_ref (it);
+                tracklist[trackcount++] = it;
             }
             deadbeef->pl_item_unref (it);
             it = next;
         }
-
-        deadbeef->pl_save_current ();
     }
     else if (ctx == DDB_ACTION_CTX_PLAYLIST) {
+        unsigned count = deadbeef->plt_get_item_count (plt, PL_MAIN);
+        tracklist = calloc (count, sizeof (DB_playItem_t *));
         DB_playItem_t *it = deadbeef->plt_get_first (plt, PL_MAIN);
         while (it) {
-            const char *uri = deadbeef->pl_find_meta (it, ":URI");
-            if (deadbeef->is_local_file (uri)) {
-                delete_and_remove_track (uri, plt, it);
-// FIXME: this dialog should allow something like "cancel" and "ignore all", then
-// it will be usable
-//                else {
-//                    GtkWidget *dlg = gtk_message_dialog_new (GTK_WINDOW (mainwin), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, _("Can't delete the file. Perhaps it doesn't exist, read-only, or part of read-only VFS, or all of the above."));
-//                    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg), uri);
-//                    gtk_window_set_title (GTK_WINDOW (dlg), _("Warning"));
-//
-//                    int response = gtk_dialog_run (GTK_DIALOG (dlg));
-//                    gtk_widget_destroy (dlg);
-//                }
+            if (trackcount == count) {
+                break;
             }
             DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
+            const char *uri = deadbeef->pl_find_meta (it, ":URI");
+            if (deadbeef->is_local_file (uri)) {
+                deadbeef->pl_item_ref (it);
+                tracklist[trackcount++] = it;
+            }
             deadbeef->pl_item_unref (it);
             it = next;
         }
-
-        deadbeef->pl_save_current ();
     }
     else if (ctx == DDB_ACTION_CTX_NOWPLAYING) {
         DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
         if (it) {
             const char *uri = deadbeef->pl_find_meta (it, ":URI");
             if (deadbeef->is_local_file (uri)) {
-                int idx = deadbeef->plt_get_item_idx (plt, it, PL_MAIN);
+                int idx = idx_current_song = deadbeef->plt_get_item_idx (plt, it, PL_MAIN);
                 if (idx != -1) {
-                    delete_and_remove_track (uri, plt, it);
+                    tracklist = calloc (1, sizeof (DB_playItem_t *));
+                    deadbeef->pl_item_ref (it);
+                    tracklist[trackcount++] = it;
                 }
             }
             deadbeef->pl_item_unref (it);
         }
     }
+
+    if (tracklist) {
+        for (unsigned i = 0; i < trackcount; i++) {
+            const char *uri = deadbeef->pl_find_meta (tracklist[i], ":URI");
+            _delete_and_remove_track_from_all_playlists (uri, plt, tracklist[i]);
+            deadbeef->pl_item_unref (tracklist[i]);
+        }
+        free (tracklist);
+        tracklist = NULL;
+    }
+    
+    if (deadbeef->conf_get_int ("gtkui.skip_deleted_songs", 0) 
+        && deadbeef->plt_get_item_idx (plt, it_current_song, PL_MAIN) == -1 
+        && deadbeef->streamer_get_current_playlist () == deadbeef->plt_get_curr_idx () 
+        && deadbeef->get_output ()->state () == OUTPUT_STATE_PLAYING) {
+        
+        if (idx_current_song != -1 
+            && deadbeef->playqueue_get_count () == 0 
+            && deadbeef->streamer_get_shuffle () == DDB_SHUFFLE_OFF) {
+            deadbeef->sendmessage (DB_EV_PLAY_NUM, 0, idx_current_song, 0);
+        }
+        else {
+            deadbeef->sendmessage(DB_EV_NEXT, 0, 0, 0);
+        }
+    }
+
+    deadbeef->pl_save_all ();
+    if (it_current_song) {
+        deadbeef->pl_item_unref (it_current_song);
+    }
     deadbeef->pl_unlock ();
     deadbeef->plt_unref (plt);
-
     deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
     return FALSE;
 }
 
 int
-action_delete_from_disk_handler (DB_plugin_action_t *act, int ctx) {
+action_delete_from_disk_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_delete_from_disk_handler_cb, (void *)(intptr_t)ctx);
     return 0;
 }
 
 typedef struct {
-    int ctx;
+    ddb_action_context_t ctx;
     ddb_playlist_t *plt;
 } trkproperties_action_ctx_t;
 
@@ -468,7 +565,7 @@ action_show_track_properties_handler_cb (void *data) {
 }
 
 int
-action_show_track_properties_handler (DB_plugin_action_t *act, int ctx) {
+action_show_track_properties_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     trkproperties_action_ctx_t *data = calloc (1, sizeof (trkproperties_action_ctx_t));
     data->ctx = ctx;
     data->plt = deadbeef->action_get_playlist ();
@@ -483,7 +580,7 @@ action_find_handler_cb (void *data) {
 }
 
 int
-action_find_handler (DB_plugin_action_t *act, int ctx) {
+action_find_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_find_handler_cb, NULL);
     return 0;
 }
@@ -497,7 +594,7 @@ action_scroll_follows_playback_handler_cb (void *data) {
 }
 
 int
-action_scroll_follows_playback_handler (DB_plugin_action_t *act, int ctx) {
+action_scroll_follows_playback_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_scroll_follows_playback_handler_cb, NULL);
     return 0;
 }
@@ -511,7 +608,7 @@ action_cursor_follows_playback_handler_cb (void *data) {
 }
 
 int
-action_cursor_follows_playback_handler (DB_plugin_action_t *act, int ctx) {
+action_cursor_follows_playback_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_cursor_follows_playback_handler_cb, NULL);
     return 0;
 }
@@ -524,7 +621,7 @@ load_playlist_thread (void *data) {
         if (!deadbeef->plt_add_files_begin (plt, 0)) {
             deadbeef->plt_clear (plt);
             int abort = 0;
-            DB_playItem_t *it = deadbeef->plt_load2 (0, plt, NULL, fname, &abort, NULL, NULL);
+            (void)deadbeef->plt_load2 (0, plt, NULL, fname, &abort, NULL, NULL);
             deadbeef->plt_save_config (plt);
             deadbeef->plt_add_files_end (plt, 0);
         }
@@ -551,7 +648,7 @@ action_load_playlist_handler_cb (void *data) {
 }
 
 int
-action_load_playlist_handler (DB_plugin_action_t *act, int ctx) {
+action_load_playlist_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_load_playlist_handler_cb, NULL);
     return 0;
 }
@@ -582,7 +679,7 @@ action_save_playlist_handler_cb (void *data) {
 }
 
 int
-action_save_playlist_handler (DB_plugin_action_t *act, int ctx) {
+action_save_playlist_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_save_playlist_handler_cb, NULL);
     return 0;
 }
@@ -597,7 +694,7 @@ action_toggle_menu_handler_cb (void *data) {
 }
 
 int
-action_toggle_menu_handler (DB_plugin_action_t *act, int ctx) {
+action_toggle_menu_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_toggle_menu_handler_cb, NULL);
     return 0;
 }
@@ -616,7 +713,7 @@ action_toggle_statusbar_handler_cb (void *data) {
 }
 
 int
-action_toggle_statusbar_handler (DB_plugin_action_t *act, int ctx) {
+action_toggle_statusbar_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_toggle_statusbar_handler_cb, NULL);
     return 0;
 }
@@ -631,7 +728,7 @@ action_toggle_designmode_handler_cb (void *data) {
 }
 
 int
-action_toggle_designmode_handler (DB_plugin_action_t *act, int ctx) {
+action_toggle_designmode_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_toggle_designmode_handler_cb, NULL);
     return 0;
 }
@@ -643,7 +740,7 @@ action_preferences_handler_cb (void *data) {
 }
 
 int
-action_preferences_handler (DB_plugin_action_t *act, int ctx) {
+action_preferences_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_preferences_handler_cb, NULL);
     return 0;
 }
@@ -686,13 +783,13 @@ action_sort_custom_handler_cb (void *data) {
 }
 
 int
-action_sort_custom_handler (DB_plugin_action_t *act, int ctx) {
+action_sort_custom_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     gdk_threads_add_idle (action_sort_custom_handler_cb, NULL);
     return 0;
 }
 
 int
-action_crop_selected_handler (DB_plugin_action_t *act, int ctx) {
+action_crop_selected_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     deadbeef->pl_crop_selected ();
     deadbeef->pl_save_current ();
     deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
@@ -709,7 +806,7 @@ action_toggle_eq_handler_cb (void *data) {
 }
 
 int
-action_toggle_eq_handler (DB_plugin_action_t *act, int ctx) {
+action_toggle_eq_handler (DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_toggle_eq_handler_cb, NULL);
     return 0;
 }
@@ -725,7 +822,7 @@ action_show_eq_handler_cb (void *data) {
 }
 
 int
-action_show_eq_handler(DB_plugin_action_t *act, int ctx) {
+action_show_eq_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_show_eq_handler_cb, NULL);
     return 0;
 }
@@ -741,7 +838,7 @@ action_hide_eq_handler_cb (void *data) {
 }
 
 int
-action_hide_eq_handler(DB_plugin_action_t *act, int ctx) {
+action_hide_eq_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_hide_eq_handler_cb, NULL);
     return 0;
 }
@@ -753,7 +850,7 @@ action_playback_loop_off_handler_cb (void *data) {
 }
 
 int
-action_playback_loop_off_handler(DB_plugin_action_t *act, int ctx) {
+action_playback_loop_off_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_playback_loop_off_handler_cb, NULL);
     return 0;
 }
@@ -765,7 +862,7 @@ action_playback_loop_single_handler_cb (void *data) {
 }
 
 int
-action_playback_loop_single_handler(DB_plugin_action_t *act, int ctx) {
+action_playback_loop_single_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_playback_loop_single_handler_cb, NULL);
     return 0;
 }
@@ -777,7 +874,7 @@ action_playback_loop_all_handler_cb (void *data) {
 }
 
 int
-action_playback_loop_all_handler(DB_plugin_action_t *act, int ctx) {
+action_playback_loop_all_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_playback_loop_all_handler_cb, NULL);
     return 0;
 }
@@ -789,7 +886,7 @@ action_playback_order_random_handler_cb (void *data) {
 }
 
 int
-action_playback_order_random_handler(DB_plugin_action_t *act, int ctx) {
+action_playback_order_random_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_playback_order_random_handler_cb, NULL);
     return 0;
 }
@@ -801,7 +898,7 @@ action_playback_order_shuffle_albums_handler_cb (void *data) {
 }
 
 int
-action_playback_order_shuffle_albums_handler(DB_plugin_action_t *act, int ctx) {
+action_playback_order_shuffle_albums_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_playback_order_shuffle_albums_handler_cb, NULL);
     return 0;
 }
@@ -813,7 +910,7 @@ action_playback_order_shuffle_handler_cb (void *data) {
 }
 
 int
-action_playback_order_shuffle_handler(DB_plugin_action_t *act, int ctx) {
+action_playback_order_shuffle_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_playback_order_shuffle_handler_cb, NULL);
     return 0;
 }
@@ -825,25 +922,25 @@ action_playback_order_linear_handler_cb (void *data) {
 }
 
 int
-action_playback_order_linear_handler(DB_plugin_action_t *act, int ctx) {
+action_playback_order_linear_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_playback_order_linear_handler_cb, NULL);
     return 0;
 }
 
 gboolean
 action_playback_order_cycle_handler_cb (void *data) {
-    int ord = deadbeef->conf_get_int ("playback.order", PLAYBACK_ORDER_LINEAR);
-    switch (ord) {
-    case PLAYBACK_ORDER_LINEAR:
+    int shuffle = deadbeef->streamer_get_shuffle ();
+    switch (shuffle) {
+    case DDB_SHUFFLE_OFF:
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (lookup_widget (mainwin, "order_shuffle")), 1);
         break;
-    case PLAYBACK_ORDER_SHUFFLE_TRACKS:
+    case DDB_SHUFFLE_TRACKS:
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (lookup_widget (mainwin, "order_shuffle_albums")), 1);
         break;
-    case PLAYBACK_ORDER_SHUFFLE_ALBUMS:
+    case DDB_SHUFFLE_ALBUMS:
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (lookup_widget (mainwin, "order_random")), 1);
         break;
-    case PLAYBACK_ORDER_RANDOM:
+    case DDB_SHUFFLE_RANDOM:
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (lookup_widget (mainwin, "order_linear")), 1);
         break;
     }
@@ -851,22 +948,22 @@ action_playback_order_cycle_handler_cb (void *data) {
 }
 
 int
-action_playback_order_cycle_handler(DB_plugin_action_t *act, int ctx) {
+action_playback_order_cycle_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_playback_order_cycle_handler_cb, NULL);
     return 0;
 }
 
 gboolean
 action_playback_loop_cycle_handler_cb (void *data) {
-    int ord = deadbeef->conf_get_int ("playback.loop", PLAYBACK_MODE_LOOP_ALL);
-    switch (ord) {
-    case PLAYBACK_MODE_LOOP_ALL:
+    int repeat = deadbeef->streamer_get_repeat ();
+    switch (repeat) {
+    case DDB_REPEAT_ALL:
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (lookup_widget (mainwin, "loop_single")), 1);
         break;
-    case PLAYBACK_MODE_LOOP_SINGLE:
+    case DDB_REPEAT_SINGLE:
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (lookup_widget (mainwin, "loop_disable")), 1);
         break;
-    case PLAYBACK_MODE_NOLOOP:
+    case DDB_REPEAT_OFF:
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (lookup_widget (mainwin, "loop_all")), 1);
         break;
     }
@@ -874,7 +971,7 @@ action_playback_loop_cycle_handler_cb (void *data) {
 }
 
 int
-action_playback_loop_cycle_handler(DB_plugin_action_t *act, int ctx) {
+action_playback_loop_cycle_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_playback_loop_cycle_handler_cb, NULL);
     return 0;
 }
@@ -886,7 +983,7 @@ action_toggle_logwindow_handler_cb (void *data) {
 }
 
 int
-action_toggle_logwindow_handler(DB_plugin_action_t *act, int ctx) {
+action_toggle_logwindow_handler(DB_plugin_action_t *act, ddb_action_context_t ctx) {
     g_idle_add (action_toggle_logwindow_handler_cb, NULL);
     return 0;
 }
