@@ -125,7 +125,7 @@ on_playbtn_clicked                     (GtkButton       *button,
 {
     // NOTE: this function is a copy of action_play_cb
     DB_output_t *output = deadbeef->get_output ();
-    if (output->state () == OUTPUT_STATE_PAUSED) {
+    if (output->state () == DDB_PLAYBACK_STATE_PAUSED) {
         ddb_playlist_t *plt = deadbeef->plt_get_curr ();
         int cur = deadbeef->plt_get_cursor (plt, PL_MAIN);
         if (cur != -1) {
@@ -202,23 +202,24 @@ on_mainwin_key_press_event             (GtkWidget       *widget,
                                         gpointer         user_data)
 {
     // local hotkeys
-    GdkModifierType consumed;
+    GdkModifierType consumed_modifiers;
     guint accel_key;
-    gdk_keymap_translate_keyboard_state (gdk_keymap_get_default (), event->hardware_keycode, event->state, 0, &accel_key, NULL, NULL, &consumed);
+
+    guint accel_mods = event->state & gtk_accelerator_get_default_mod_mask ();
+
+    GdkDisplay *display = gtk_widget_get_display (widget);
+
+    gdk_keymap_translate_keyboard_state (gdk_keymap_get_for_display (display), event->hardware_keycode, accel_mods & (~GDK_SHIFT_MASK), 0, &accel_key, NULL, NULL, &consumed_modifiers);
     if (accel_key == GDK_ISO_Left_Tab)
         accel_key = GDK_Tab;
-    int mods = event->state & gtk_accelerator_get_default_mod_mask ();
-    mods &= ~(consumed&~GDK_SHIFT_MASK);
-    int lower = gdk_keyval_to_lower (accel_key);
-    if (lower != accel_key) {
-        accel_key = lower;
-    }
     trace ("pressed: keycode: %x, mods: %x, hw: %x, translated: %x\n", event->keyval, mods, event->hardware_keycode, accel_key);
 
+    accel_mods &= ~(consumed_modifiers&~GDK_SHIFT_MASK);
+
     DB_plugin_t *hkplug = deadbeef->plug_get_for_id ("hotkeys");
     if (hkplug) {
         int ctx;
-        DB_plugin_action_t *act = ((DB_hotkeys_plugin_t *)hkplug)->get_action_for_keycombo (accel_key, mods, 0, &ctx);
+        DB_plugin_action_t *act = ((DB_hotkeys_plugin_t *)hkplug)->get_action_for_keycombo (accel_key, accel_mods, 0, &ctx);
         if (act && act->callback2) {
             trace ("executing action %s in ctx %d\n", act->name, ctx);
             act->callback2 (act, ctx);
@@ -237,8 +238,9 @@ void
 on_order_linear_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    deadbeef->conf_set_int ("playback.order", PLAYBACK_ORDER_LINEAR);
-    deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
+    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) {
+        deadbeef->streamer_set_shuffle (DDB_SHUFFLE_OFF);
+    }
 }
 
 
@@ -246,24 +248,27 @@ void
 on_order_shuffle_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    deadbeef->conf_set_int ("playback.order", PLAYBACK_ORDER_SHUFFLE_TRACKS);
-    deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
+    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) {
+        deadbeef->streamer_set_shuffle (DDB_SHUFFLE_TRACKS);
+    }
 }
 
 void
 on_order_shuffle_albums_activate       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    deadbeef->conf_set_int ("playback.order", PLAYBACK_ORDER_SHUFFLE_ALBUMS);
-    deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
+    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) {
+        deadbeef->streamer_set_shuffle (DDB_SHUFFLE_ALBUMS);
+    }
 }
 
 void
 on_order_random_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    deadbeef->conf_set_int ("playback.order", PLAYBACK_ORDER_RANDOM);
-    deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
+    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) {
+        deadbeef->streamer_set_shuffle (DDB_SHUFFLE_RANDOM);
+    }
 }
 
 
@@ -271,8 +276,9 @@ void
 on_loop_all_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    deadbeef->conf_set_int ("playback.loop", 0);
-    deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
+    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) {
+        deadbeef->streamer_set_repeat (DDB_REPEAT_ALL);
+    }
 }
 
 
@@ -280,8 +286,9 @@ void
 on_loop_single_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    deadbeef->conf_set_int ("playback.loop", 2);
-    deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
+    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) {
+        deadbeef->streamer_set_repeat (DDB_REPEAT_SINGLE);
+    }
 }
 
 
@@ -289,8 +296,9 @@ void
 on_loop_disable_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    deadbeef->conf_set_int ("playback.loop", 1);
-    deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
+    if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) {
+        deadbeef->streamer_set_repeat (DDB_REPEAT_OFF);
+    }
 }
 
 gboolean
@@ -800,3 +808,4 @@ on_log_window_key_press_event          (GtkWidget       *widget,
     }
     return FALSE;
 }
+
