@@ -1,20 +1,26 @@
 /*
-    DeaDBeeF - The Ultimate Music Player
-    Copyright (C) 2009-2013 Alexey Yakovenko <waker@users.sourceforge.net>
+    DeaDBeeF -- the music player
+    Copyright (C) 2009-2015 Alexey Yakovenko and other contributors
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+    This software is provided 'as-is', without any express or implied
+    warranty.  In no event will the authors be held liable for any damages
+    arising from the use of this software.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    Permission is granted to anyone to use this software for any purpose,
+    including commercial applications, and to alter it and redistribute it
+    freely, subject to the following restrictions:
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    1. The origin of this software must not be misrepresented; you must not
+     claim that you wrote the original software. If you use this software
+     in a product, an acknowledgment in the product documentation would be
+     appreciated but is not required.
+
+    2. Altered source versions must be plainly marked as such, and must not be
+     misrepresented as being the original software.
+
+    3. This notice may not be removed or altered from any source distribution.
 */
+
 #include <gtk/gtk.h>
 
 
@@ -156,59 +162,6 @@ void
 on_searchentry_changed                 (GtkEditable     *editable,
                                         gpointer         user_data);
 
-gboolean
-on_searchheader_button_press_event     (GtkWidget       *widget,
-                                        GdkEventButton  *event,
-                                        gpointer         user_data);
-
-gboolean
-on_searchheader_button_release_event   (GtkWidget       *widget,
-                                        GdkEventButton  *event,
-                                        gpointer         user_data);
-
-gboolean
-on_searchheader_configure_event        (GtkWidget       *widget,
-                                        GdkEventConfigure *event,
-                                        gpointer         user_data);
-
-gboolean
-on_searchheader_expose_event           (GtkWidget       *widget,
-                                        GdkEventExpose  *event,
-                                        gpointer         user_data);
-
-gboolean
-on_searchheader_motion_notify_event    (GtkWidget       *widget,
-                                        GdkEventMotion  *event,
-                                        gpointer         user_data);
-
-gboolean
-on_searchlist_button_press_event       (GtkWidget       *widget,
-                                        GdkEventButton  *event,
-                                        gpointer         user_data);
-
-gboolean
-on_searchlist_configure_event          (GtkWidget       *widget,
-                                        GdkEventConfigure *event,
-                                        gpointer         user_data);
-
-gboolean
-on_searchlist_expose_event             (GtkWidget       *widget,
-                                        GdkEventExpose  *event,
-                                        gpointer         user_data);
-
-gboolean
-on_searchlist_scroll_event             (GtkWidget       *widget,
-                                        GdkEvent        *event,
-                                        gpointer         user_data);
-
-void
-on_searchscroll_value_changed          (GtkRange        *range,
-                                        gpointer         user_data);
-
-void
-on_searchlist_realize                  (GtkWidget       *widget,
-                                        gpointer         user_data);
-
 gboolean
 on_header_button_press_event           (GtkWidget       *widget,
                                         GdkEventButton  *event,
@@ -234,7 +187,6 @@ on_header_motion_notify_event          (GtkWidget       *widget,
                                         GdkEventMotion  *event,
                                         gpointer         user_data);
 
-
 void
 on_playlist_load_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -326,10 +278,6 @@ void
 on_select_all1_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_remove1_activate                    (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_find_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -339,14 +287,6 @@ void
 on_help1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_playhscroll_value_changed           (GtkRange        *range,
-                                        gpointer         user_data);
-
-void
-on_searchhscroll_value_changed         (GtkRange        *range,
-                                        gpointer         user_data);
-
 gboolean
 on_helpwindow_key_press_event          (GtkWidget       *widget,
                                         GdkEventKey     *event,
@@ -432,10 +372,6 @@ void
 on_custom_activate                     (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_remove_column_activate              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_pref_alsa_resampling_clicked        (GtkButton       *button,
                                         gpointer         user_data);
@@ -540,10 +476,6 @@ on_remove_from_playback_queue1_activate
                                         (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_remove2_activate                    (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_properties1_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -580,14 +512,6 @@ on_trackproperties_key_press_event     (GtkWidget       *widget,
                                         GdkEventKey     *event,
                                         gpointer         user_data);
 
-void
-on_add_column_activate                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_edit_column_activate                (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 gboolean
 on_trackproperties_delete_event        (GtkWidget       *widget,
                                         GdkEvent        *event,
@@ -898,11 +822,11 @@ on_hide_tray_icon_toggled              (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
 void
-on_embolden_current_toggled            (GtkToggleButton *togglebutton,
+on_hide_delete_from_disk_toggled       (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
 void
-on_hide_delete_from_disk_toggled       (GtkToggleButton *togglebutton,
+on_skip_deleted_songs_toggled          (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
 void
@@ -1276,11 +1200,6 @@ void
 on_hotkeys_defaults_clicked            (GtkButton       *button,
                                         gpointer         user_data);
 
-gboolean
-on_searchwin_key_press_event           (GtkWidget       *widget,
-                                        GdkEventKey     *event,
-                                        gpointer         user_data);
-
 void
 on_auto_size_columns_toggled           (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
@@ -1305,3 +1224,202 @@ on_mainwin_button_press_event          (GtkWidget       *widget,
 void
 on_enable_shift_jis_recoding_toggled   (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
+
+void
+on_listview_playing_text_color_set     (GtkColorButton  *colorbutton,
+                                        gpointer         user_data);
+
+void
+on_listview_group_text_color_set       (GtkColorButton  *colorbutton,
+                                        gpointer         user_data);
+
+void
+on_listview_group_text_font_set        (GtkFontButton   *fontbutton,
+                                        gpointer         user_data);
+
+void
+on_listview_text_font_set              (GtkFontButton   *fontbutton,
+                                        gpointer         user_data);
+
+void
+on_listview_playing_text_bold_toggled  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_listview_playing_text_italic_toggled
+                                        (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_listview_selected_text_bold_toggled (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_listview_selected_text_italic_toggled
+                                        (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_tabstrip_selected_text_color_set    (GtkColorButton  *colorbutton,
+                                        gpointer         user_data);
+
+void
+on_tabstrip_playing_bold_toggled       (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_tabstrip_playing_italic_toggled     (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_tabstrip_selected_bold_toggled      (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_tabstrip_selected_italic_toggled    (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_tabstrip_text_font_set              (GtkFontButton   *fontbutton,
+                                        gpointer         user_data);
+
+void
+on_tabstrip_playing_text_color_set     (GtkColorButton  *colorbutton,
+                                        gpointer         user_data);
+
+void
+on_listview_column_text_color_set      (GtkColorButton  *colorbutton,
+                                        gpointer         user_data);
+
+void
+on_listview_column_text_font_set       (GtkFontButton   *fontbutton,
+                                        gpointer         user_data);
+
+gboolean
+on_prefwin_key_press_event             (GtkWidget       *widget,
+                                        GdkEventKey     *event,
+                                        gpointer         user_data);
+
+void
+on_trkpropertis_edit_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_edit_in_place_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_remove_activate       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_cut_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_copy_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_paste_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_capitalize_activate   (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_clean_up_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_format_from_other_fields_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_add_new_field_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_paste_fields_activate (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_automatically_fill_values_activate
+                                        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_crop_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_trkproperties_edit_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_pref_replaygain_source_mode_changed (GtkComboBox     *combobox,
+                                        gpointer         user_data);
+
+void
+on_pref_replaygain_processing_changed  (GtkComboBox     *combobox,
+                                        gpointer         user_data);
+
+void
+on_view_log_activate                   (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_log_clear_clicked                   (GtkButton       *button,
+                                        gpointer         user_data);
+
+gboolean
+on_log_window_key_press_event          (GtkWidget       *widget,
+                                        GdkEventKey     *event,
+                                        gpointer         user_data);
+
+void
+on_display_seltime_toggled             (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_pref_pluginlist_row_activated       (GtkTreeView     *treeview,
+                                        GtkTreePath     *path,
+                                        GtkTreeViewColumn *column,
+                                        gpointer         user_data);
+void
+on_listview_group_spacing_value_changed
+                                        (GtkSpinButton   *spinbutton,
+                                        gpointer         user_data);
+
+void
+on_comboboxentry_direct_sr_changed     (GtkComboBox     *combobox,
+                                        gpointer         user_data);
+
+void
+on_comboboxentry_sr_mult_48_changed    (GtkComboBox     *combobox,
+                                        gpointer         user_data);
+
+void
+on_comboboxentry_sr_mult_44_changed    (GtkComboBox     *combobox,
+                                        gpointer         user_data);
+
+void
+on_checkbutton_sr_override_toggled     (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+
+void
+on_checkbutton_dependent_sr_toggled    (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_minimize_on_startup_clicked         (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_move_to_trash_clicked               (GtkButton       *button,
+                                        gpointer         user_data);
