@@ -273,20 +273,30 @@ gtkui_run_preferences_dlg (void) {
 
     // preamp without rg
     set_scale("global_preamp", deadbeef->conf_get_int ("replaygain.preamp_without_rg", 0));
+    
     // dsp
     dsp_setup_init (prefwin);
 
+    // minimize_on_startup
+    set_toggle_button("minimize_on_startup", deadbeef->conf_get_int ("gtkui.start_hidden", 0));
+    
     // close_send_to_tray
     set_toggle_button("pref_close_send_to_tray", deadbeef->conf_get_int ("close_send_to_tray", 0));
 
     // hide tray icon
     set_toggle_button("hide_tray_icon", deadbeef->conf_get_int ("gtkui.hide_tray_icon", 0));
 
+    // move_to_trash
+    set_toggle_button("move_to_trash", deadbeef->conf_get_int ("gtkui.move_to_trash", 1));
+
     // mmb_delete_playlist
     set_toggle_button("mmb_delete_playlist", deadbeef->conf_get_int ("gtkui.mmb_delete_playlist", 1));
 
     // hide_delete_from_disk
     set_toggle_button("hide_delete_from_disk", deadbeef->conf_get_int ("gtkui.hide_remove_from_disk", 0));
+    
+    // play next song when currently played is deleted
+    set_toggle_button("skip_deleted_songs", deadbeef->conf_get_int ("gtkui.skip_deleted_songs", 0));
 
     // auto-rename playlist from folder name
     set_toggle_button("auto_name_playlist_from_folder", deadbeef->conf_get_int ("gtkui.name_playlist_from_folder", 1));
@@ -585,6 +595,28 @@ on_global_preamp_value_changed     (GtkRange        *range,
     deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
 }
 
+void
+on_minimize_on_startup_clicked     (GtkButton       *button,
+                                   gpointer         user_data)
+{
+    int active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+    deadbeef->conf_set_int ("gtkui.start_hidden", active);
+    if (active == 1) {
+        set_toggle_button("hide_tray_icon", 0);
+        deadbeef->conf_set_int ("gtkui.hide_tray_icon", 0);
+    }
+    deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
+}
+
+void
+on_move_to_trash_clicked               (GtkButton       *button,
+                                        gpointer         user_data)
+{
+    int active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
+    deadbeef->conf_set_int ("gtkui.move_to_trash", active);
+    deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
+}
+
 void
 on_pref_close_send_to_tray_clicked     (GtkButton       *button,
                                         gpointer         user_data)
@@ -600,6 +632,10 @@ on_hide_tray_icon_toggled              (GtkToggleButton *togglebutton,
 {
     int active = gtk_toggle_button_get_active (togglebutton);
     deadbeef->conf_set_int ("gtkui.hide_tray_icon", active);
+    if (active == 1) {
+        set_toggle_button("minimize_on_startup", 0);
+        deadbeef->conf_set_int ("gtkui.start_hidden", 0);
+    }
     deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
 }
 
@@ -1066,6 +1102,14 @@ on_hide_delete_from_disk_toggled       (GtkToggleButton *togglebutton,
     deadbeef->conf_set_int ("gtkui.hide_remove_from_disk", active);
 }
 
+void
+on_skip_deleted_songs_toggled          (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+    int active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (togglebutton));
+    deadbeef->conf_set_int ("gtkui.skip_deleted_songs", active);
+}
+
 void
 on_titlebar_format_playing_changed     (GtkEditable     *editable,
                                         gpointer         user_data)
