@@ -22,9 +22,8 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+    #include <config.h>
 #endif
-#include <vorbis/vorbisfile.h>
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
@@ -33,6 +32,13 @@
 #include <stdbool.h>
 #include "../../deadbeef.h"
 #include "../liboggedit/oggedit.h"
+#if TREMOR
+    #include <tremor/ivorbisfile.h>
+    #define FIXED_POINT 1
+#else
+    #include <vorbis/vorbisfile.h>
+#endif
+#include "../../strdupa.h"
 
 #define min(x,y) ((x)<(y)?(x):(y))
 #define max(x,y) ((x)>(y)?(x):(y))
@@ -46,8 +52,22 @@
 #define CVORBIS_ENDIANNESS 0
 #endif
 
-#define RG_REFERENCE_LOUDNESS -1
-#define DELIMITER "\n - \n"
+static const char *tag_rg_names[] = {
+    "REPLAYGAIN_ALBUM_GAIN",
+    "REPLAYGAIN_ALBUM_PEAK",
+    "REPLAYGAIN_TRACK_GAIN",
+    "REPLAYGAIN_TRACK_PEAK",
+    NULL
+};
+
+// replaygain key names in deadbeef internal metadata
+static const char *ddb_internal_rg_keys[] = {
+    ":REPLAYGAIN_ALBUMGAIN",
+    ":REPLAYGAIN_ALBUMPEAK",
+    ":REPLAYGAIN_TRACKGAIN",
+    ":REPLAYGAIN_TRACKPEAK",
+    NULL
+};
 
 static DB_decoder_t plugin;
 static DB_functions_t *deadbeef;
@@ -58,6 +78,7 @@ typedef struct {
     int cur_bit_stream;
     float next_update;
     DB_playItem_t *it;
+    int set_bitrate;
     const DB_playItem_t *new_track;
     uint8_t *channel_map;
 } ogg_info_t;
@@ -86,57 +107,10 @@ cvorbis_ftell (void *datasource) {
     return deadbeef->ftell (datasource);
 }
 
-static const char
-*gain_tag_name(const int tag_enum)
-{
-    switch(tag_enum) {
-        case DDB_REPLAYGAIN_ALBUMGAIN:
-            return "REPLAYGAIN_ALBUM_GAIN";
-        case DDB_REPLAYGAIN_ALBUMPEAK:
-            return "REPLAYGAIN_ALBUM_PEAK";
-        case DDB_REPLAYGAIN_TRACKGAIN:
-            return "REPLAYGAIN_TRACK_GAIN";
-        case DDB_REPLAYGAIN_TRACKPEAK:
-            return "REPLAYGAIN_TRACK_PEAK";
-        case RG_REFERENCE_LOUDNESS:
-            return "REPLAYGAIN_REFERENCE_LOUDNESS";
-        default:
-            return "";
-    }
-}
-
-static bool
-is_special_tag(const char *tag) {
-    return !strcasecmp(tag, gain_tag_name(DDB_REPLAYGAIN_ALBUMGAIN)) ||
-           !strcasecmp(tag, gain_tag_name(DDB_REPLAYGAIN_ALBUMPEAK)) ||
-           !strcasecmp(tag, gain_tag_name(DDB_REPLAYGAIN_TRACKGAIN)) ||
-           !strcasecmp(tag, gain_tag_name(DDB_REPLAYGAIN_TRACKPEAK)) ||
-           !strcasecmp(tag, gain_tag_name(RG_REFERENCE_LOUDNESS));
-}
-
-static bool
-add_meta(DB_playItem_t *it, const char *key, const char *value)
-{
-    const char *old_value = deadbeef->pl_find_meta(it, key);
-    if (old_value) {
-        char *new_value = malloc(strlen(old_value) + strlen(DELIMITER) + strlen(value) + 1);
-        if (new_value) {
-            sprintf(new_value, "%s"DELIMITER"%s", old_value, value);
-            deadbeef->pl_replace_meta(it, key, new_value);
-            free(new_value);
-            return true;
-        }
-    }
-    else {
-        deadbeef->pl_add_meta(it, key, value);
-        return true;
-    }
-}
-
 static bool
 replaygain_tag(DB_playItem_t *it, const int tag_enum, const char *tag, const char *value)
 {
-    if (strcasecmp(gain_tag_name(tag_enum), tag))
+    if (strcasecmp(tag_rg_names[tag_enum], tag))
         return false;
 
     deadbeef->pl_set_item_replaygain (it, tag_enum, atof(value));
@@ -155,17 +129,22 @@ update_vorbis_comments (DB_playItem_t *it, OggVorbis_File *vorbis_file, const in
     for (int i = 0; i < vc->comments; i++) {
         char *tag = strdup(vc->user_comments[i]);
         char *value;
-        if (tag && (value = strchr(tag, '='))) {
+        if (tag && (value = strchr(tag, '='))
+#ifdef ANDROID
+            && strlen (value) < 4000
+#endif
+            ) {
             *value++ = '\0';
             if (!replaygain_tag(it, DDB_REPLAYGAIN_ALBUMGAIN, tag, value) &&
                 !replaygain_tag(it, DDB_REPLAYGAIN_ALBUMPEAK, tag, value) &&
                 !replaygain_tag(it, DDB_REPLAYGAIN_TRACKGAIN, tag, value) &&
-                !replaygain_tag(it, DDB_REPLAYGAIN_TRACKPEAK, tag, value)) {
-                if (!strcasecmp(tag, gain_tag_name(RG_REFERENCE_LOUDNESS)))
-                    deadbeef->pl_replace_meta(it, ":REPLAYGAIN_REFERENCE_LOUDNESS", value);
-                else
-                    add_meta(it, oggedit_map_tag(tag, "tag2meta"), value);
-                }
+                !replaygain_tag(it, DDB_REPLAYGAIN_TRACKPEAK, tag, value)
+                && strcasecmp (tag, "METADATA_BLOCK_PICTURE")
+                && *value) {
+                    deadbeef->pl_append_meta(it, oggedit_map_tag(tag, "tag2meta"), value);
+            }
+        }
+        if (tag) {
             free(tag);
         }
     }
@@ -180,7 +159,7 @@ update_vorbis_comments (DB_playItem_t *it, OggVorbis_File *vorbis_file, const in
         deadbeef->plt_modified (plt);
         deadbeef->plt_unref (plt);
     }
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, 0, 0);
+    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
 
     return 0;
 }
@@ -212,26 +191,61 @@ cvorbis_seek_sample (DB_fileinfo_t *_info, int sample) {
         }
         deadbeef->pl_unlock ();
     }
-    sample += info->it->startsample;
+    int64_t startsample = deadbeef->pl_item_get_startsample (info->it);
+    sample += startsample;
     trace ("vorbis: seek to sample %d\n", sample);
     int res = ov_pcm_seek (&info->vorbis_file, sample);
     if (res != 0 && res != OV_ENOSEEK) {
         trace ("vorbis: error %x seeking to sample %d\n", res, sample);
         return -1;
     }
-    int tell = ov_pcm_tell (&info->vorbis_file);
+    int64_t tell = ov_pcm_tell (&info->vorbis_file);
     if (tell != sample) {
         trace ("vorbis: failed to do sample-accurate seek (%d->%d)\n", sample, tell);
     }
     trace ("vorbis: seek successful\n")
-    _info->readpos = (float)(sample - info->it->startsample)/_info->fmt.samplerate;
+    _info->readpos = (float)(sample - startsample)/_info->fmt.samplerate;
     info->next_update = -2;
     return 0;
 }
 
+static ogg_info_t *
+cvorbis_open_int (uint32_t hints) {
+    ogg_info_t *info = calloc(1, sizeof(ogg_info_t));
+    if (info) {
+        info->info.plugin = &plugin;
+#if FIXED_POINT
+        info->info.fmt.bps = 16;
+#else
+        info->info.fmt.is_float = 1;
+        info->info.fmt.bps = 32;
+#endif
+        info->set_bitrate = hints & DDB_DECODER_HINT_NEED_BITRATE;
+    }
+    return info;
+}
+
 static DB_fileinfo_t *
 cvorbis_open (uint32_t hints) {
-    return calloc(1, sizeof (ogg_info_t));
+    return (DB_fileinfo_t *)cvorbis_open_int(hints);
+}
+
+static DB_fileinfo_t *
+cvorbis_open2 (uint32_t hints, DB_playItem_t *it) {
+    ogg_info_t *info = cvorbis_open_int(hints);
+    if (!info) {
+        return NULL;
+    }
+
+    deadbeef->pl_lock();
+    const char *uri = strdupa (deadbeef->pl_find_meta (it, ":URI"));
+    deadbeef->pl_unlock();
+    info->info.file = deadbeef->fopen(uri);
+    if (!info->info.file) {
+        trace("cvorbis_open2 failed to open file %s\n", uri);
+    }
+
+    return &info->info;
 }
 
 static int
@@ -240,15 +254,18 @@ cvorbis_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     ogg_info_t *info = (ogg_info_t *)_info;
     info->new_track = info->it = it;
     deadbeef->pl_item_ref (it);
-    deadbeef->pl_replace_meta (it, "!FILETYPE", "OggVorbis");
 
-    deadbeef->pl_lock ();
-    info->info.file = deadbeef->fopen (deadbeef->pl_find_meta (it, ":URI"));
-    deadbeef->pl_unlock ();
     if (!info->info.file) {
-        trace ("ogg: failed to open file %s\n", deadbeef->pl_find_meta (it, ":URI"));
-        return -1;
+        deadbeef->pl_lock ();
+	const char *uri = strdupa (deadbeef->pl_find_meta (it, ":URI"));
+        deadbeef->pl_unlock ();
+        info->info.file = deadbeef->fopen (uri);
+        if (!info->info.file) {
+            trace ("cvorbis_init failed to open file %s\n", uri);
+            return -1;
+        }
     }
+
     if (info->info.file->vfs->is_streaming () && deadbeef->fgetlength (info->info.file) == -1) {
         ov_callbacks ovcb = {
             .read_func = cvorbis_fread,
@@ -268,7 +285,6 @@ cvorbis_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
         if (plt) {
             deadbeef->plt_unref (plt);
         }
-        deadbeef->pl_replace_meta (it, "!FILETYPE", "OggVorbis");
     }
     else
     {
@@ -288,12 +304,12 @@ cvorbis_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
     }
 //    _info->readpos = 0;
     if (info->info.file->vfs->is_streaming ()) {
-        info->it->startsample = 0;
+        deadbeef->pl_item_set_startsample (info->it, 0);
         if (deadbeef->pl_get_item_duration (it) < 0) {
-            it->endsample = -1;
+            deadbeef->pl_item_set_endsample (info->it, -1);
         }
         else {
-            it->endsample = ov_pcm_total (&info->vorbis_file, -1)-1;
+            deadbeef->pl_item_set_endsample (it, ov_pcm_total (&info->vorbis_file, -1)-1);
         }
         if (update_vorbis_comments (it, &info->vorbis_file, -1))
             return -1;
@@ -312,15 +328,19 @@ cvorbis_init (DB_fileinfo_t *_info, DB_playItem_t *it) {
         trace ("vorbis: bad samplerate\n");
         return -1;
     }
-    _info->plugin = &plugin;
-    _info->fmt.bps = 16;
-    _info->fmt.samplerate = vi->rate;
+    _info->fmt.samplerate = (int)vi->rate;
     _info->fmt.channels = vi->channels;
     info->channel_map = oggedit_vorbis_channel_map(vi->channels);
     for (int i = 0; i < _info->fmt.channels; i++) {
         _info->fmt.channelmask |= 1 << i;
     }
 
+    deadbeef->pl_set_meta_int (it, ":TRACKNUM", 0);
+    deadbeef->pl_set_meta_int (it, ":CHANNELS", vi->channels);
+    deadbeef->pl_set_meta_int (it, ":SAMPLERATE", (int)vi->rate);
+    deadbeef->pl_replace_meta (it, "!FILETYPE", "Ogg Vorbis");
+    info->cur_bit_stream = -1;
+
     return 0;
 }
 
@@ -349,7 +369,7 @@ cvorbis_free (DB_fileinfo_t *_info) {
 static void send_event(DB_playItem_t *it, const int event_enum)
 {
     ddb_event_track_t *event = (ddb_event_track_t *)deadbeef->event_alloc(event_enum);
-    if (event->track = it)
+    if ((event->track = it))
         deadbeef->pl_item_ref(event->track);
     deadbeef->event_send((ddb_event_t *)event, 0, 0);
 }
@@ -357,30 +377,33 @@ static void send_event(DB_playItem_t *it, const int event_enum)
 static bool
 new_streaming_link(ogg_info_t *info, const int new_link)
 {
-    if (info->cur_bit_stream == 0 && new_link != 1)
+    // Typically the files with multiple links are split into multiple tracks,
+    // and we don't want the file's properties to change while it's playing.
+    // However, we want this for network streams.
+    if (!info->info.file->vfs->is_streaming () || new_link < 0) {
         return false;
+    }
 
-    trace("Streaming link changed from %d to %d\n", info->cur_bit_stream, new_link);
-    deadbeef->pl_set_meta_int(info->it, ":TRACKNUM", new_link);
-    update_vorbis_comments(info->it, &info->vorbis_file, new_link);
-    send_event(info->it, DB_EV_SONGSTARTED);
-    send_event(info->it, DB_EV_TRACKINFOCHANGED);
-    deadbeef->sendmessage(DB_EV_PLAYLISTCHANGED, 0, 0, 0);
+    trace ("Streaming link changed from %d to %d\n", info->cur_bit_stream, new_link);
+    update_vorbis_comments (info->it, &info->vorbis_file, new_link);
+    send_event (info->it, DB_EV_SONGSTARTED);
+    send_event (info->it, DB_EV_TRACKINFOCHANGED);
+    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
     info->cur_bit_stream = new_link;
 
     vorbis_info *vi = ov_info (&info->vorbis_file, new_link);
-    if (vi && info->info.fmt.channels != vi->channels || info->info.fmt.samplerate != vi->rate) {
-        // Streamer can't do this, so re-init the stream
-        trace("Stream channel count changed from %d to %d\n", info->info.fmt.channels, vi->channels);
-        trace("Stream channel count changed from %d to %d\n", info->info.fmt.samplerate, vi->rate);
-        deadbeef->sendmessage(DB_EV_PAUSE, 0, 0, 0);
-        deadbeef->sendmessage(DB_EV_TOGGLE_PAUSE, 0, 0, 0);
+    if (vi && (info->info.fmt.channels != vi->channels || info->info.fmt.samplerate != vi->rate)) {
+        info->info.fmt.samplerate = (int)vi->rate;
+        info->info.fmt.channels = vi->channels;
+        deadbeef->pl_set_meta_int (info->it, ":CHANNELS", vi->channels);
+        deadbeef->pl_set_meta_int (info->it, ":SAMPLERATE", (int)vi->rate);
         return true;
     }
 
     return false;
 }
 
+#if FIXED_POINT
 static void
 map_channels(int16_t *dest, const int16_t *src, const int16_t *src_end, const uint8_t *map, const int channel_count)
 {
@@ -390,6 +413,7 @@ map_channels(int16_t *dest, const int16_t *src, const int16_t *src_end, const ui
         dest += channel_count;
     }
 }
+#endif
 
 static bool
 is_playing_track(const DB_playItem_t *it)
@@ -412,27 +436,37 @@ cvorbis_read (DB_fileinfo_t *_info, char *buffer, int bytes_to_read) {
     }
 
     /* Don't read past the end of a sub-track */
-    if (deadbeef->pl_get_item_flags(info->it) & DDB_IS_SUBTRACK) {
-        const ogg_int64_t bytes_left = (info->it->endsample - ov_pcm_tell(&info->vorbis_file)) * 2 * _info->fmt.channels;
-        if (bytes_left < bytes_to_read)
-            bytes_to_read = bytes_left;
+    int samples_to_read = bytes_to_read / sizeof(float) / _info->fmt.channels;
+    int64_t endsample = deadbeef->pl_item_get_endsample (info->it);
+    if (endsample > 0) {
+        const ogg_int64_t samples_left = endsample - ov_pcm_tell(&info->vorbis_file);
+        if (samples_left < samples_to_read) {
+            samples_to_read = (int)samples_left;
+            bytes_to_read = samples_to_read * sizeof(float) * _info->fmt.channels;
+        }
     }
 
     /* Read until we have enough bytes to satisfy streamer, or there are none left */
+    int bytes_read = 0;
+    int ret = OV_HOLE;
+
+#if FIXED_POINT
     char map_buffer[info->channel_map ? bytes_to_read : 0];
     char *ptr = info->channel_map ? map_buffer : buffer;
-    int ret = OV_HOLE;
-    int bytes_read = 0;
     while ((ret > 0 || ret == OV_HOLE) && bytes_read < bytes_to_read)
     {
         int new_link = -1;
-        ret=ov_read (&info->vorbis_file, ptr+bytes_read, bytes_to_read-bytes_read, CVORBIS_ENDIANNESS, 2, 1, &new_link);
-
+#if TREMOR
+        ret=ov_read(&info->vorbis_file, ptr+bytes_read, bytes_to_read-bytes_read, &new_link);
+#else
+        ret=ov_read(&info->vorbis_file, ptr+bytes_read, bytes_to_read-bytes_read, CVORBIS_ENDIANNESS, 2, 1, &new_link);
+#endif
         if (ret < 0) {
             trace("cvorbis_read: ov_read returned %d\n", ret);
         }
-        else if (new_link != info->cur_bit_stream && !ov_seekable(&info->vorbis_file) && new_streaming_link(info, new_link)) {
+        else if (new_link != info->cur_bit_stream && new_streaming_link(info, new_link)) {
             bytes_read = bytes_to_read;
+            break;
         }
         else {
             bytes_read += ret;
@@ -442,16 +476,48 @@ cvorbis_read (DB_fileinfo_t *_info, char *buffer, int bytes_to_read) {
     }
 
     if (info->channel_map)
-        map_channels((int16_t *)buffer, (int16_t *)map_buffer, (int16_t *)(ptr+bytes_read), info->channel_map, _info->fmt.channels);
+        map_channels((int16_t *)buffer, (int16_t *)map_buffer, (int16_t *)ptr, info->channel_map, _info->fmt.channels);
+#else
+    int samples_read = 0;
+    while (samples_read < samples_to_read && (ret > 0 || ret == OV_HOLE))
+    {
+        float **pcm = NULL;
+        int new_link = -1;
+        ret = (int)ov_read_float(&info->vorbis_file, &pcm, samples_to_read-samples_read, &new_link);
 
-    _info->readpos = (float)(ov_pcm_tell(&info->vorbis_file) - info->it->startsample) / _info->fmt.samplerate;
-    if (_info->readpos > info->next_update) {
-        const int rate = ov_bitrate_instant(&info->vorbis_file) / 1000;
+        if (ret < 0) {
+            trace("cvorbis_read: ov_read returned %d\n", ret);
+        }
+        else if (new_link != info->cur_bit_stream && new_streaming_link(info, new_link)) {
+            samples_read = samples_to_read;
+            break;
+        }
+        else if (ret > 0) {
+            float *ptr = (float *)buffer + samples_read*_info->fmt.channels;
+            for (int channel = 0; channel < _info->fmt.channels; channel++, ptr++) {
+                const float *pcm_channel = pcm[info->channel_map ? info->channel_map[channel] : channel];
+                for (int sample = 0; sample < ret; sample++) {
+                    ptr[sample*_info->fmt.channels] = pcm_channel[sample];
+                }
+            }
+            samples_read += ret;
+        }
+
+//        trace("cvorbis_read got %d samples towards %d bytes (%d samples still required)\n", ret, bytes_to_read, samples_to_read-samples_read);
+    }
+    bytes_read = samples_read * sizeof(float) * _info->fmt.channels;
+#endif
+
+    int64_t startsample = deadbeef->pl_item_get_startsample (info->it);
+    _info->readpos = (float)(ov_pcm_tell(&info->vorbis_file) - startsample) / _info->fmt.samplerate;
+    if (info->set_bitrate && _info->readpos > info->next_update) {
+        const int rate = (int)ov_bitrate_instant(&info->vorbis_file) / 1000;
         if (rate > 0) {
             deadbeef->streamer_set_bitrate(rate);
             info->next_update = info->next_update <= 0 ? info->next_update + 1 : _info->readpos + 5;
         }
     }
+
 //    trace("cvorbis_read returning %d bytes out of %d\n", bytes_read, bytes_to_read);
     return bytes_read;
 }
@@ -508,31 +574,38 @@ cvorbis_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
     }
 
     long nstreams = ov_streams (&vorbis_file);
-    int currentsample = 0;
+    int64_t currentsample = 0;
     for (int stream = 0; stream < nstreams; stream++) {
         const vorbis_info *vi = ov_info (&vorbis_file, stream);
         if (!vi) {
             trace ("vorbis: ov_info failed for file %s stream %d\n", fname, stream);
             continue;
         }
-        float duration = ov_time_total (&vorbis_file, stream);
-        int totalsamples = ov_pcm_total (&vorbis_file, stream);
+#if TREMOR
+        const float duration = ov_time_total(&vorbis_file, stream) / 1000.0;
+#else
+        const float duration = ov_time_total(&vorbis_file, stream);
+#endif
+        int64_t totalsamples = ov_pcm_total (&vorbis_file, stream);
 
         DB_playItem_t *it = deadbeef->pl_item_alloc_init (fname, plugin.plugin.id);
         deadbeef->pl_set_meta_int (it, ":TRACKNUM", stream);
         deadbeef->plt_set_item_duration (plt, it, duration);
         if (nstreams > 1) {
-            it->startsample = currentsample;
-            it->endsample = currentsample + totalsamples - 1;
+            deadbeef->pl_item_set_startsample (it, currentsample);
+            deadbeef->pl_item_set_endsample (it, currentsample + totalsamples - 1);
             deadbeef->pl_set_item_flags (it, DDB_IS_SUBTRACK);
         }
 
         if (update_vorbis_comments (it, &vorbis_file, stream))
             continue;
-        int samplerate = vi->rate;
+        int samplerate = (int)vi->rate;
 
-        const off_t start_offset = sample_offset(&vorbis_file, it->startsample-1);
-        const off_t end_offset = sample_offset(&vorbis_file, it->endsample);
+        int64_t startsample = deadbeef->pl_item_get_startsample (it);
+        int64_t endsample = deadbeef->pl_item_get_endsample (it);
+
+        const off_t start_offset = sample_offset(&vorbis_file, startsample-1);
+        const off_t end_offset = sample_offset(&vorbis_file, endsample);
         char *filetype = NULL;
         const off_t stream_size = oggedit_vorbis_stream_info(deadbeef->fopen(fname), start_offset, end_offset, &filetype);
         if (filetype) {
@@ -544,36 +617,18 @@ cvorbis_insert (ddb_playlist_t *plt, DB_playItem_t *after, const char *fname) {
             deadbeef->pl_set_meta_int(it, ":BITRATE", 8.f * samplerate * stream_size / totalsamples / 1000);
         }
         set_meta_ll (it, ":FILE_SIZE", fsize);
-        deadbeef->pl_add_meta (it, ":BPS", "16");
+//        deadbeef->pl_add_meta (it, ":BPS", "32");
         deadbeef->pl_set_meta_int (it, ":CHANNELS", vi->channels);
         deadbeef->pl_set_meta_int (it, ":SAMPLERATE", samplerate);
 //        deadbeef->pl_set_meta_int (it, ":BITRATE", ov_bitrate (&vorbis_file, stream)/1000);
 
         if (nstreams == 1) {
-            DB_playItem_t *cue = deadbeef->plt_insert_cue (plt, after, it, totalsamples, samplerate);
+            DB_playItem_t *cue = deadbeef->plt_process_cue (plt, after, it,  totalsamples, samplerate);
             if (cue) {
                 deadbeef->pl_item_unref (it);
-                deadbeef->pl_item_unref (cue);
                 ov_clear (&vorbis_file);
                 return cue;
             }
-
-            deadbeef->pl_lock ();
-            const char *cuesheet_meta = deadbeef->pl_find_meta (it, "cuesheet");
-            if (cuesheet_meta) {
-                trace("Embedded cuesheet found for %s\n", fname);
-                const char *last_sheet = strstr(cuesheet_meta, DELIMITER);
-                const char *cuesheet = last_sheet ? last_sheet + strlen(DELIMITER) : cuesheet_meta;
-                cue = deadbeef->plt_insert_cue_from_buffer (plt, after, it, cuesheet, strlen (cuesheet), totalsamples, samplerate);
-                if (cue) {
-                    deadbeef->pl_unlock ();
-                    deadbeef->pl_item_unref (it);
-                    deadbeef->pl_item_unref (cue);
-                    ov_clear (&vorbis_file);
-                    return cue;
-                }
-            }
-            deadbeef->pl_unlock ();
         }
         else {
             currentsample += totalsamples;
@@ -602,14 +657,15 @@ cvorbis_read_metadata (DB_playItem_t *it) {
     vorbis_info *vi = NULL;
 
     deadbeef->pl_lock ();
-    fp = deadbeef->fopen (deadbeef->pl_find_meta (it, ":URI"));
+    const char *uri = strdupa (deadbeef->pl_find_meta (it, ":URI"));
     deadbeef->pl_unlock ();
+    fp = deadbeef->fopen (uri);
     if (!fp) {
-        trace ("cvorbis_read_metadata: failed to fopen %s\n", deadbeef->pl_find_meta (it, ":URI"));
+        trace ("cvorbis_read_metadata: failed to fopen %s\n", uri);
         return -1;
     }
     if (fp->vfs->is_streaming ()) {
-        trace ("cvorbis_read_metadata: failed to fopen %s\n", deadbeef->pl_find_meta (it, ":URI"));
+        trace ("cvorbis_read_metadata: failed to fopen %s\n", uri);
         return -1;
     }
     ov_callbacks ovcb = {
@@ -626,7 +682,7 @@ cvorbis_read_metadata (DB_playItem_t *it) {
     int tracknum = deadbeef->pl_find_meta_int (it, ":TRACKNUM", -1);
     vi = ov_info (&vorbis_file, tracknum);
     if (!vi) {
-        trace ("cvorbis_read_metadata: failed to ov_open %s\n", deadbeef->pl_find_meta (it, ":URI"));
+        trace ("cvorbis_read_metadata: failed to ov_open %s\n", uri);
         ov_clear (&vorbis_file);
         return -1;
     }
@@ -641,68 +697,53 @@ cvorbis_read_metadata (DB_playItem_t *it) {
 }
 
 static void
-split_tag(vorbis_comment *tags, const char *key, const char *value)
+split_tag(vorbis_comment *tags, const char *key, const char *value, int valuesize)
 {
-    if (key && value) {
-        const char *p;
-        while (p = strstr(value, DELIMITER)) {
-            char v[p - value + 1];
-            strncpy(v, value, p-value);
-            v[p-value] = 0;
-            vorbis_comment_add_tag(tags, key, v);
-            value = p + strlen(DELIMITER);
-        }
-
+    while (valuesize > 0) {
         vorbis_comment_add_tag(tags, key, value);
+        size_t l = strlen (value) + 1;
+        value += l;
+        valuesize -= l;
     }
 }
 
-static void
-merge_gain_tag(DB_playItem_t *it, vorbis_comment *vc, vorbis_comment *tags, const int tag_enum, const char *pattern, const int min, const int max)
+static vorbis_comment *
+tags_list(DB_playItem_t *it, OggVorbis_File *vorbis_file)
 {
-    const char *key = gain_tag_name(tag_enum);
-
-    char *end;
-    const char *meta_value = deadbeef->pl_find_meta(it, key);
-    const float value = meta_value ? strtof(meta_value, &end) : 0;
-    if (meta_value && end != meta_value && value > min && value < max) {
-        char tag_value[10];
-        sprintf(tag_value, pattern, value);
-        vorbis_comment_add_tag(tags, key, tag_value);
-    }
-    else {
-        const char *tag_value = vorbis_comment_query(vc, key, 0);
-        if (tag_value)
-             vorbis_comment_add_tag(tags, key, tag_value);
-    }
-}
-
-static vorbis_comment
-*tags_list(DB_playItem_t *it, OggVorbis_File *vorbis_file)
-{
-    vorbis_comment *vc = ov_comment (vorbis_file, -1);
-    if (!vc)
-        return NULL;
-
-    vorbis_comment *tags = calloc(1, sizeof(vorbis_comment));
+    vorbis_comment *tags = calloc (1, sizeof (vorbis_comment));
     if (!tags)
         return NULL;
 
     deadbeef->pl_lock ();
-    merge_gain_tag(it, vc, tags, DDB_REPLAYGAIN_ALBUMGAIN, "%0.2f dB", -100, 100);
-    merge_gain_tag(it, vc, tags, DDB_REPLAYGAIN_ALBUMPEAK, "%0.8f", 0, 2);
-    merge_gain_tag(it, vc, tags, DDB_REPLAYGAIN_TRACKGAIN, "%0.2f dB", -100, 100);
-    merge_gain_tag(it, vc, tags, DDB_REPLAYGAIN_TRACKPEAK, "%0.8f", 0, 2);
-    merge_gain_tag(it, vc, tags, RG_REFERENCE_LOUDNESS, "%0.1f dB", 0, 128);
-    for (DB_metaInfo_t *m = deadbeef->pl_get_metadata_head(it); m; m = m->next) {
-        char *key = strdup(m->key);
-        if (key && key[0] != ':' && key[0] != '!' && !is_special_tag(key)) {
-            split_tag(tags, oggedit_map_tag(key, "meta2tag"), m->value);
-            free(key);
+    for (DB_metaInfo_t *m = deadbeef->pl_get_metadata_head (it); m; m = m->next) {
+        if (strchr (":!_", m->key[0])) {
+            break;
         }
+        char *key = strdupa (m->key);
+        split_tag (tags, oggedit_map_tag (key, "meta2tag"), m->value, m->valuesize);
     }
     deadbeef->pl_unlock ();
 
+    // add replaygain values
+    for (int n = 0; ddb_internal_rg_keys[n]; n++) {
+        if (deadbeef->pl_find_meta (it, ddb_internal_rg_keys[n])) {
+            float value = deadbeef->pl_get_item_replaygain (it, n);
+            char s[100];
+            // https://wiki.hydrogenaud.io/index.php?title=ReplayGain_2.0_specification#Metadata_format
+            switch (n) {
+            case DDB_REPLAYGAIN_ALBUMGAIN:
+            case DDB_REPLAYGAIN_TRACKGAIN:
+                snprintf (s, sizeof (s), "%.2f dB", value);
+                break;
+            case DDB_REPLAYGAIN_ALBUMPEAK:
+            case DDB_REPLAYGAIN_TRACKPEAK:
+                snprintf (s, sizeof (s), "%.6f", value);
+                break;
+            }
+            split_tag (tags, tag_rg_names[n], s, (int)strlen (s)+1);
+        }
+    }
+
     return tags;
 }
 
@@ -744,13 +785,22 @@ cvorbis_write_metadata (DB_playItem_t *it) {
     const off_t file_size = oggedit_write_vorbis_metadata (deadbeef->fopen(fname), fname, 0, stream_size, tags->comments, tags->user_comments);
     vorbis_comment_clear(tags);
     free(tags);
+
+    int res = 0;
+
     if (file_size <= 0) {
         trace ("cvorbis_write_metadata: failed to write tags to %s, code %d\n", fname, file_size);
-        return -1;
+        res = -1;
     }
 
-    set_meta_ll(it, ":FILE_SIZE", (int64_t)file_size);
-    return cvorbis_read_metadata(it);
+    ov_clear (&vorbis_file);
+
+    if (!res) {
+        set_meta_ll(it, ":FILE_SIZE", (int64_t)file_size);
+        res = cvorbis_read_metadata(it);
+    }
+
+    return res;
 }
 
 
@@ -758,16 +808,15 @@ static const char * exts[] = { "ogg", "ogx", "oga", NULL };
 
 // define plugin interface
 static DB_decoder_t plugin = {
-    .plugin.api_vmajor = 1,
-    .plugin.api_vminor = 0,
+    DB_PLUGIN_SET_API_VERSION
     .plugin.version_major = 1,
     .plugin.version_minor = 0,
     .plugin.type = DB_PLUGIN_DECODER,
     .plugin.id = "stdogg",
-    .plugin.name = "OggVorbis decoder",
-    .plugin.descr = "OggVorbis decoder using standard xiph.org libraries",
+    .plugin.name = "Ogg Vorbis decoder",
+    .plugin.descr = "Ogg Vorbis decoder using standard xiph.org libraries",
     .plugin.copyright =
-    "OggVorbis plugin for DeaDBeeF\n"
+    "Ogg Vorbis plugin for DeaDBeeF\n"
     "Copyright (C) 2009-2014 Alexey Yakovenko et al.\n"
     "\n"
     "vcedit.c\n"
@@ -827,6 +876,7 @@ static DB_decoder_t plugin = {
     .plugin.start = vorbis_start,
     .plugin.stop = vorbis_stop,
     .open = cvorbis_open,
+    .open2 = cvorbis_open2,
     .init = cvorbis_init,
     .free = cvorbis_free,
     .read = cvorbis_read,
